VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsMarkPositions"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'================================================================================================================
'
'Copyright (c)2004 - Klaus Dahlenburg, INCOMM GmbH, Ger
'
'This program is free software; you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation; either version 2 of the License, or
'(at your option) any later version.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

'Eine deutsche bersetzung der GNU General Public License kann unter
'"http://www.gnu.de/gpl-ger.html" eingesehen werden.
'================================================================================================================
'
' changes:
'   29.05.2008 105 Florian Tree
'           - changed Form, implemented Delete function
'
'================================================================================================================
'
' -------------------
' Diese Klasse kapselt die Zugriffe und das Setzen von Markern an zu findenden Balloons
' Die Verbindung zu SolidEdge und das Draftdokument kann ebenfalls von dieser Klasse
' aufgebaut werden. Wird die Methode setBases() nicht aufgerufen, so stellt die Klasse
' die notwendige Verbindung her.
'
Option Explicit
'
' SolidEdge-Objekte
Private objApp As SolidEdgeFramework.Application
Private objDraft As SolidEdgeDraft.DraftDocument
Private objActiveSheet As SolidEdgeDraft.Sheet
Private objGroups As SolidEdgeFrameworkSupport.Groups
Private objGroup As SolidEdgeFrameworkSupport.Group
Private lcntMarker As Long
Private lcntDelMarker As Long
'
Private sFehler As String

' -------
' Rueckgabe einer Fehlermeldung
' -------
Public Property Get getError() As String
    getError = sFehler
End Property

' -------
' Rueckgabe des Markierunszaehler
' -------
Public Property Get getCount() As Long
    getCount = lcntMarker
End Property
' -------
' Rueckgabe des Lsch Markierunszaehler
' -------
Public Property Get getDelCount() As Long
    getDelCount = lcntDelMarker
End Property

' -------
' Setzen der Objekte fuer SolidEdge und das Draft-Dokument
' -------
Public Function setBases(pobjApp As SolidEdgeFramework.Application, _
                         pobjDraft As SolidEdgeDraft.DraftDocument) As Boolean
    setBases = True
    Set objApp = pobjApp
    Set objDraft = pobjDraft
    If (pobjApp Is Nothing Or pobjDraft Is Nothing) Then
        setBases = False
        sFehler = "Internal: setBases() called and at least one argument not set"
    End If
    '
End Function
' -----------
' Markieren von Positionsnummern (Balloons)
' Argument: die zu suchende Positionsnummer als Long
'
' Der Rueckgabewert ist: True = Position(en) markiert
'
' Beruecksichtigt werden nur Balloons, die einem Part (Item) zugeordnet sind
' Callouts, und Leader werden uebergangen
' ------------
Public Function searchBalloons(ByVal sMarkPos As String) As Boolean
    
    Dim i As Long
    Dim sBalText As String
    '
    sFehler = vbNullString
    lcntMarker = 0
    '
    ' Turn on error trapping.
    On Error Resume Next

    ' Verbindung zu Solid Edge Draft aufbauen
    If objApp Is Nothing Then
        Set objApp = GetObject(, "SolidEdge.Application")
        If Err.Number <> 0 Then
            sFehler = "Solid Edge ist nicht gestartet!" & vbNewLine _
                      & "Fehlercode: " & Err.Number & " Fehlertext: " & Err.Description
            Exit Function
        End If
    End If
    ' ist Draft-Datei aktiv?
    If objApp.ActiveDocumentType <> igDraftDocument Then
        ' keine Draft datei geffnet.
        sFehler = "Das aktuelle Dokument ist kein Draft-Dokument!"
        Exit Function
    End If
    
    'Open the document.
    Set objDraft = objApp.ActiveDocument
    If Err.Number Then
        'keine SE-Datei geffnet
        sFehler = "Fehler beim Zugriff auf das Draft-Dokument!" & vbNewLine _
                  & "Fehlercode: " & Err.Number & " Fehlertext: " & Err.Description
        Exit Function
    End If
    
    '
    Set objActiveSheet = objDraft.ActiveSheet
    objDraft.SelectSet.RemoveAll
 
    ' First check balloons inside of groups
    If objActiveSheet.Groups.Count > 0 Then
        lcntMarker = 0
        Call chkGroup(objActiveSheet.Groups, lcntMarker, sMarkPos)
    End If
    '
    '
    If objDraft.ActiveSheet.Balloons.Count = 0 Then
        If lcntMarker = 0 Then
            searchBalloons = False
        End If
        Exit Function
    End If
    '
    On Error Resume Next  ' activate error handler
    '
    With objActiveSheet.Balloons
        For i = .Count To 1 Step -1
            If .Item(i).Type = igBalloon Then
            ' ------------
               ' uncomment and connect to IF with an AND if
               ' only circled items should be considered valid
               ' .Item(i).BalloonType = igDimBalloonCircle Then
               '
                If .Item(i).Callout = 0 Then      ' the one we want?
                    sBalText = .Item(i).BalloonText  ' may throw an error if not associated
                    If Err.Number = 0 And sBalText = sMarkPos Then        ' = 0 balloon associated with an item
                        Call setMarker(.Item(i), sBalText)
                        lcntMarker = lcntMarker + 1
                    Else
                        Err.Clear
                    End If  '  /* If Err.Number */
                    '
                End If  '  /* If .Item(i).Callout */
                '
            End If  '  /* .Item(i).Type */
            '
            ' get next balloon
        Next
        '
    End With
    '
    On Error GoTo 0            ' error handler deaktivieren
    Set objActiveSheet = Nothing
    searchBalloons = lcntMarker > 0
    '
End Function
' -----------------------------
' Set a marker to the matching position. The marker can be freely
' moved around because it's not associated with the object. Thisway we
' avoid the grey-framed parts-list
' -----------------------------
Private Sub setMarker(objBal As SolidEdgeFrameworkSupport.Balloon, sPos As String)
'
'
Dim dx As Double
Dim dy As Double
Dim dz As Double
Dim objTerm As Object
Dim bTerm As Boolean
Dim objBall As SolidEdgeFrameworkSupport.Balloon
Dim objSbal As SolidEdgeFrameworkSupport.DimStyle
Dim objAttrSet As Object
'
    ' get the object this balloon is connected to
    Call objBal.GetTerminator(objTerm, dx, dy, dz, bTerm)
    ' now place our marker at that position
    Set objBall = objActiveSheet.Balloons.Add(dx - 0.01, dy, dz)
    ' get the style for the new balloon
    Set objSbal = objBall.Style
    ' modify some values
    objSbal.DrivenColor = RGB(255, 0, 0)
    objSbal.Font = "Arial Black"
    objSbal.FontStyle = igDimStyleFontBold
    objSbal.Height = 0.01
    objSbal.LineWidth = 0.00125
    ' set the shape of our marke
    objBall.BalloonSize = 3
    objBall.BalloonDirection = igDimBalloonDirectionTop
    objBall.BalloonType = igDimBalloonNSided
    objBall.BalloonSides = 6
    ' no true position and no leader
    objBall.Callout = 0
    'objBall.CalloutBorder = True
    objBall.Leader = False
    
    ' set some custom Attributes so we can identify it for aut. deletion
    ' Create the attribute set called 'MarkPositions'.
    Set objAttrSet = objBall.AttributeSets.Add("MarkPositions")
    ' Add the string attribute called 'FindBallon'.  This sample
    ' just adds a single attribute to the attribute set, but any number
    ' of attributes can be added and they can be of many different types
    ' including: seByte, seInteger, seLong, seSingle, seDouble, seCurrency
    ' seDate, seStringANSI, seStringUnicode, or seBoolean.  These constants
    ' are defined in the Solid Edge Constants type library
    objAttrSet.Add "FindBallon", seBoolean
    objAttrSet.FindBallon = True
    
    objAttrSet.Add "PosNr", seStringANSI
    objAttrSet.PosNr = sPos
    
    ' now insert the text
    objBall.BalloonText = sPos
    '
    Set objBall = Nothing
    Set objSbal = Nothing
    Set objTerm = Nothing
End Sub

Private Sub Class_Initialize()
'
End Sub

Private Sub Class_Terminate()
'
    Set objApp = Nothing
    Set objDraft = Nothing
    Set objActiveSheet = Nothing
    Set objGroups = Nothing
    Set objGroup = Nothing
'
End Sub

' --------------------------------
' Funktion: recursivly check groups inside a group to find a matching balloon
'           if a group does contain one or more groups
'           we call ourself with the groups object
'
' Return:   True = no error, false otherwise
' Externals: objGroup As SolidEdgeFrameworkSupport.Group
' Note:     argument pCnt will reflect the number of ballons found
'           and must therefore be passed in as ByRef
' --------------------------------
Private Sub chkGroup(ByVal pGroups As SolidEdgeFrameworkSupport.Groups, _
                          ByRef pCnt As Long, _
                          sTheItem As String)
'
Static lCount As Long
Static sBalText As String
'
    sFehler = vbNullString
    On Error Resume Next                    ' enable error handler
    '
    For Each objGroup In pGroups
        '
        ' first get all balloons from ungrouped
        ' views that a group may contain
        For lCount = 1 To objGroup.Count
            If objGroup.Item(lCount).Type = igBalloon Then
                ' uncomment and connect to IF with an AND if
                ' only circled items should be considered valid
                ' objGroup.Item(lCount).BalloonType = igDimBalloonCircle Then
                '
                If objGroup.Item(lCount).Callout = 0 Then      ' the one we want?
                    sBalText = objGroup.Item(lCount).BalloonText  ' may throw an error if not associated
                    ' = 0 balloon associated with an item and matching
                    If Err.Number = 0 And sBalText = sTheItem Then
                        Call setMarker(objGroup.Item(lCount), sTheItem)
                    Else
                        Err.Clear ' ignore error
                    End If
                End If
            End If
        Next
        '
        ' in case the group does contain
        ' other groups examine them now
        If objGroup.HasNested > 0 Then
            Call chkGroup(objGroup.Groups, pCnt, sTheItem)
        End If
    Next
    '
    ' exit
    Set objGroup = Nothing
    Exit Sub
'
End Sub

' -----------
' Lschen von vorher gesetzten von Positionsnummern (Balloons)
'
' Der Rueckgabewert ist: True = Position(en) markiert
'
' Beruecksichtigt werden nur Balloons, die ein Custom AttributeSet "MarkPositions" mit dem Attribute "FindBallon" haben
' Callouts, und Leader werden uebergangen
' ------------
Public Function delBalloons() As Boolean
    
    Dim i As Long
    Dim sBalText As String
    '
    sFehler = vbNullString
    lcntDelMarker = 0
    '
    ' Turn on error trapping.
    On Error Resume Next

    ' Verbindung zu Solid Edge Draft aufbauen
    If objApp Is Nothing Then
        Set objApp = GetObject(, "SolidEdge.Application")
        If Err.Number <> 0 Then
            sFehler = "Solid Edge ist nicht gestartet!" & vbNewLine _
                      & "Fehlercode: " & Err.Number & " Fehlertext: " & Err.Description
            Exit Function
        End If
    End If
    ' ist Draft-Datei aktiv?
    If objApp.ActiveDocumentType <> igDraftDocument Then
        ' keine Draft datei geffnet.
        sFehler = "Das aktuelle Dokument ist kein Draft-Dokument!"
        Exit Function
    End If
    
    'Open the document.
    Set objDraft = objApp.ActiveDocument
    If Err.Number Then
        'keine SE-Datei geffnet
        sFehler = "Fehler beim Zugriff auf das Draft-Dokument!" & vbNewLine _
                  & "Fehlercode: " & Err.Number & " Fehlertext: " & Err.Description
        Exit Function
    End If
    
    '
    Set objActiveSheet = objDraft.ActiveSheet
    objDraft.SelectSet.RemoveAll
 
'    ' First check balloons inside of groups
'    If objActiveSheet.Groups.Count > 0 Then
'        lcntMarker = 0
'        Call chkGroup(objActiveSheet.Groups, lcntMarker, sMarkPos)
'    End If
    '
    '
    If objDraft.ActiveSheet.Balloons.Count = 0 Then
        If lcntMarker = 0 Then
            delBalloons = False
        End If
        Exit Function
    End If
    '
    On Error Resume Next  ' activate error handler
    '
    With objActiveSheet.Balloons
        For i = .Count To 1 Step -1
            If .Item(i).Type = igBalloon Then
            ' ------------
               ' uncomment and connect to IF with an AND if
               ' only circled items should be considered valid
               ' .Item(i).BalloonType = igDimBalloonCircle Then
               '
                If .Item(i).Callout = 0 Then      ' the one we want?
                
                    If .Item(i).IsAttributeSetPresent("MarkPositions") Then
                        ' we found an search Ballon so delete it
                        If .Item(i).FindBallon = True Then
                            .Item(i).Delete
                            lcntDelMarker = lcntDelMarker + 1
                        End If
                    End If
                
                    '
                End If  '  /* If .Item(i).Callout */
                '
            End If  '  /* .Item(i).Type */
            '
            ' get next balloon
        Next
        '
    End With
    '
    On Error GoTo 0            ' error handler deaktivieren
    Set objActiveSheet = Nothing
    delBalloons = lcntDelMarker > 0
    '
End Function


